import parse from 'parse-link-header';
import semver from 'semver';
import promptTerminal from './promptWrapper.js';
import { handleRes } from './readmeAPIFetch.js';
function specOptions(specList, parsedDocs, currPage, totalPages) {
    const specs = specList.map(s => {
        return {
            description: `API Definition ID: ${s._id}`, // eslint-disable-line no-underscore-dangle
            title: s.title,
            value: s._id, // eslint-disable-line no-underscore-dangle
        };
    });
    if (parsedDocs?.prev?.page) {
        specs.push({
            description: 'Go to the previous page',
            title: `< Prev (page ${currPage - 1} of ${totalPages})`,
            value: 'prev',
        });
    }
    if (parsedDocs?.next?.page) {
        specs.push({
            description: 'Go to the next page',
            title: `Next (page ${currPage + 1} of ${totalPages}) >`,
            value: 'next',
        });
    }
    return specs;
}
const updateOasPrompt = (specList, parsedDocs, currPage, totalPages, getSpecs) => [
    {
        type: 'select',
        name: 'specId',
        message: 'Select your desired file to update',
        choices: specOptions(specList, parsedDocs, currPage, totalPages),
        async format(spec) {
            if (spec === 'prev') {
                try {
                    const newSpecs = await getSpecs(`${parsedDocs?.prev?.url || ''}`);
                    const newParsedDocs = parse(newSpecs.headers.get('link'));
                    const newSpecList = await handleRes(newSpecs);
                    const { specId } = await promptTerminal(updateOasPrompt(newSpecList, newParsedDocs, currPage - 1, totalPages, getSpecs));
                    return specId;
                }
                catch (e) {
                    return null;
                }
            }
            else if (spec === 'next') {
                try {
                    const newSpecs = await getSpecs(`${parsedDocs?.next?.url || ''}`);
                    const newParsedDocs = parse(newSpecs.headers.get('link'));
                    const newSpecList = await handleRes(newSpecs);
                    const { specId } = await promptTerminal(updateOasPrompt(newSpecList, newParsedDocs, currPage + 1, totalPages, getSpecs));
                    return specId;
                }
                catch (e) {
                    return null;
                }
            }
            return spec;
        },
    },
];
export function createOasPrompt(specList, parsedDocs, totalPages, getSpecs) {
    return [
        {
            type: 'select',
            name: 'option',
            message: 'Would you like to update an existing OAS file or create a new one?',
            choices: [
                { title: 'Update existing', value: 'update' },
                { title: 'Create a new spec', value: 'create' },
            ],
            async format(picked) {
                if (picked === 'update') {
                    const { specId } = await promptTerminal(updateOasPrompt(specList, parsedDocs, 1, totalPages, getSpecs));
                    return specId;
                }
                return picked;
            },
        },
    ];
}
/**
 * Series of prompts to construct a version object,
 * used in our `versions:create` and `versions:update` commands
 */
export function versionPrompt(
/** list of versions, used for prompt about which version to fork */
versionList, 
/** existing version if we're performing an update */
existingVersion) {
    return [
        {
            // only runs for versions:create command
            type: existingVersion ? null : 'select',
            name: 'from',
            message: 'Which version would you like to fork from?',
            choices: versionList.map(v => {
                return {
                    title: v.version,
                    value: v.version,
                };
            }),
        },
        {
            // only runs for versions:update command
            type: !existingVersion ? null : 'text',
            name: 'newVersion',
            message: 'What should the version be renamed to?',
            hint: '1.0.0',
            validate(val) {
                // allow empty string, in which case the version won't be renamed
                if (!val)
                    return true;
                return semver.valid(semver.coerce(val)) ? true : 'Please specify a semantic version.';
            },
        },
        {
            // if the user is already updating the main version
            // we skip this question since it must remain the main version
            type: existingVersion?.is_stable ? null : 'confirm',
            name: 'is_stable',
            message: 'Should this be the main version for your project?',
        },
        {
            type: 'confirm',
            name: 'is_beta',
            message: 'Should this version be in beta?',
        },
        {
            type: (prev, values) => {
                // if user wants this version to be the main version
                // it can't also be hidden.
                return values.is_stable || existingVersion?.is_stable ? null : 'confirm';
            },
            name: 'is_hidden',
            message: 'Should this version be hidden?',
        },
        {
            type: (prev, values) => {
                // if user wants this version to be the main version
                // it can't also be deprecated.
                return values.is_stable || existingVersion?.is_stable ? null : 'confirm';
            },
            name: 'is_deprecated',
            message: 'Should this version be deprecated?',
        },
    ];
}
